@@ -46,13 +46,8 @@ OPTIONAL_PROPERTIES = {
                               "driver should use virtual media USB or floppy "
                               "device for passing configuration information "
                               "to the ramdisk. Defaults to False. Optional."),
-    'kernel_append_params': _("Additional kernel parameters to pass down to "
-                              "instance kernel. These parameters can be "
-                              "consumed by the kernel or by the applications "
-                              "by reading /proc/cmdline. Mind severe cmdline "
-                              "size limit. Overrides "
-                              "[redfish]/kernel_append_params ironic "
-                              "option."),
+    'kernel_append_params': driver_utils.KERNEL_APPEND_PARAMS_DESCRIPTION %
+    {'option_group': 'redfish'},
     'bootloader': _("URL or Glance UUID  of the EFI system partition "
                     "image containing EFI boot loader. This image will be "
                     "used by ironic when building UEFI-bootable ISO "
@@ -96,9 +91,14 @@ def _parse_driver_info(node):
     :raises: InvalidParameterValue, if any of the parameters have invalid
         value.
     """
+    mode = deploy_utils.rescue_or_deploy_mode(node)
+    if not deploy_utils.needs_agent_ramdisk(node, mode=mode):
+        # Ramdisk deploy does not need an agent, nor does it support any other
+        # options. Skipping.
+        return {'can_provide_config': False}
+
     d_info = node.driver_info
 
-    mode = deploy_utils.rescue_or_deploy_mode(node)
     iso_param = f'{mode}_iso'
     iso_ref = driver_utils.get_agent_iso(node, deprecated_prefix='redfish',
                                          mode=mode)
@@ -278,7 +278,7 @@ def _has_vmedia_device(managers, boot_device, inserted=None):
         inserted status.
     :return: The device that could be found or False.
     """
-    if isinstance(boot_device, str):
+    if not isinstance(boot_device, list):
         boot_device = [boot_device]
 
     for dev in boot_device:
@@ -407,18 +407,34 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         d_info = _parse_deploy_info(node)
         deploy_utils.validate_image_properties(task, d_info)
 
-    def _validate_vendor(self, task):
+    def _validate_vendor(self, task, managers):
+        """Validates vendor specific requirements for the task's node.
+
+        :param task: a TaskManager instance containing the node to act on.
+        :param managers: Redfish managers for Redfish system associated
+            with node.
+        :raises: InvalidParameterValue if vendor not supported
+        """
         vendor = task.node.properties.get('vendor')
         if not vendor:
             return
 
         if 'Dell' in vendor.split():
+            # Check if iDRAC fw >= 6.00.00.00 that supports virtual media boot
+            bmc_manager = [m for m in managers
+                           if m.manager_type == sushy.MANAGER_TYPE_BMC]
+            if bmc_manager:
+                fwv = bmc_manager[0].firmware_version.split('.')
+                if int(fwv[0]) >= 6:
+                    return
             raise exception.InvalidParameterValue(
                 _("The %(iface)s boot interface is not suitable for node "
-                  "%(node)s with vendor %(vendor)s, use "
+                  "%(node)s with vendor %(vendor)s and BMC version %(fwv)s, "
+                  "upgrade to 6.00.00.00 or newer or use "
                   "idrac-redfish-virtual-media instead")
-                % {'iface': task.node.boot_interface,
-                   'node': task.node.uuid, 'vendor': vendor})
+                % {'iface': task.node.get_interface('boot'),
+                   'node': task.node.uuid, 'vendor': vendor,
+                   'fwv': bmc_manager[0].firmware_version})
 
     def validate(self, task):
         """Validate the deployment information for the task's node.
@@ -431,7 +447,6 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         :raises: InvalidParameterValue on malformed parameter(s)
         :raises: MissingParameterValue on missing parameter(s)
         """
-        self._validate_vendor(task)
         self._validate_driver_info(task)
         self._validate_instance_info(task)
 
@@ -468,19 +483,14 @@ class RedfishVirtualMediaBoot(base.BootInterface):
             operation failed on the node.
         """
         node = task.node
-        # NOTE(TheJulia): If this method is being called by something
-        # aside from deployment, clean and rescue, such as conductor takeover,
-        # we should treat this as a no-op and move on otherwise we would
-        # modify the state of the node due to virtual media operations.
-        if node.provision_state not in (states.DEPLOYING,
-                                        states.CLEANING,
-                                        states.RESCUING,
-                                        states.INSPECTING):
+        if not driver_utils.need_prepare_ramdisk(node):
             return
 
         d_info = _parse_driver_info(node)
         managers = redfish_utils.get_system(task.node).managers
 
+        self._validate_vendor(task, managers)
+
         if manager_utils.is_fast_track(task):
             if _has_vmedia_device(managers, sushy.VIRTUAL_MEDIA_CD,
                                   inserted=True):
@@ -639,11 +649,7 @@ class RedfishVirtualMediaBoot(base.BootInterface):
 
         del managers
 
-        persistent = True
-        if node.driver_info.get('force_persistent_boot_device',
-                                'Default') == 'Never':
-            persistent = False
-        self._set_boot_device(task, boot_devices.CDROM, persistent=persistent)
+        self._set_boot_device(task, boot_devices.CDROM, persistent=True)
 
         LOG.debug("Node %(node)s is set to permanently boot from "
                   "%(device)s", {'node': task.node.uuid,
